import random
import argparse
import os
from utilities_immuneml.yaml_tools import *

parser = argparse.ArgumentParser()
parser.add_argument('--simulation_yaml', help='Input simulation yaml file', required=True)
parser.add_argument('--n_kmers', help='Number of kmers to generate as integer', type=int, required=True)
parser.add_argument('--kmer_size', help='size of kmers as integer', type=int, required=True)
parser.add_argument('--dirname', help='directory where the specification file to write', required=True)
args = parser.parse_args()


def generate_random_kmer(kmer_size):
    alphabet = list("ACDEFGHIKLMNPQRSTVWY")
    alphabet.sort()
    kmer = "".join(random.choices(alphabet, k=kmer_size))
    return kmer


def generate_multiple_random_kmer_specs(n_kmers, kmer_size):
    spec_dict = {}
    n_kmer_labels = ["_".join(["kmer", "n", str(i), "size", str(kmer_size)]) for i in range(1, n_kmers + 1)]
    for label in n_kmer_labels:
        spec_dict[label] = {'instantiation': 'GappedKmer', 'seed': generate_random_kmer(kmer_size)}
    return spec_dict


def populate_multiple_random_kmers(sim_yaml_file, spec_dict):
    simulation_yaml_obj = parse_yaml(sim_yaml_file)
    simulation_yaml_obj['definitions']['motifs'] = spec_dict
    return simulation_yaml_obj


def execute():
    spec_dict = generate_multiple_random_kmer_specs(args.n_kmers, args.kmer_size)
    filename = "_".join([str(args.n_kmers), "kmers", "sized", str(args.kmer_size)])
    filepath = os.path.join(args.dirname, filename + ".yaml")
    sim_yaml_obj = populate_multiple_random_kmers(args.simulation_yaml,spec_dict)
    write_yaml_file(sim_yaml_obj, args.dirname, filepath)

if __name__ == "__main__":
    print(generate_random_kmer(12))