import os
import random
import math
import time
import argparse


parser = argparse.ArgumentParser()
parser.add_argument('--rep_size', help='Desired number of sequences in the random repertoire', required=True, type=int)
parser.add_argument('--directory', help='The source directory containing a set of repertoires, from which a random '
                                        'repertoire is to be generated (based on random sampling of sequences)', required=True)
parser.add_argument('--outfile', help='name of the output repertoire file', required=True)
args = parser.parse_args()


def extract_random_lines(filename, size, new_file):
    with open(filename, 'r') as f:
        f.seek(0, 2)
        filesize = f.tell()
        random_set = sorted(random.sample(range(filesize), size))
        for i in range(size):
            f.seek(random_set[i])
            f.readline()
            new_file.append(f.readline())


def generate_random_repertoire_from_dir(rep_size, directory, outfile):
    file_list = [fn for fn in os.listdir(directory) if fn.endswith(".tsv")]
    directory_size = len(file_list)
    assert directory_size > 0, "the input directory does not contain repertoire files with tsv extension"
    newfile = []
    with open(os.path.join(directory, file_list[0])) as fn:
        first_line = fn.readline()
        newfile.append(first_line)
    file_size = len(newfile)
    draw_ratio = math.floor(rep_size / directory_size)
    if draw_ratio < 10:
        draw_size = 5
    else:
        print("using a source directory of size: ----", directory_size)
        draw_size = math.floor(((rep_size / directory_size) / 100.0) * 100) - math.floor(((rep_size / directory_size) / 100.0) * 100) % 10
    while file_size <= rep_size:
        filename = random.choice(file_list)
        file_path = os.path.join(directory, filename)
        # print("reading file:",file_path)
        extract_random_lines(file_path, draw_size, new_file=newfile)
        file_size = len(newfile)
        # print("current file size is:",file_size)
    with open(outfile, "w") as outfn:
        for line in newfile:
            outfn.write(str(line))


def execute():
    start_time = time.time()
    print("desired repertoire size", args.rep_size)
    # print("desired repertoire size", args.rep_size)
    generate_random_repertoire_from_dir(rep_size=args.rep_size, directory=args.directory,
                                        outfile=args.outfile)
    print("--- One random repertoire generation completed in %s seconds ---" % (time.time() - start_time))

# if __name__ == "__main__":
#     generate_random_repertoire_from_dir(rep_size=10000, directory="/Users/kanduric/Downloads/emerson-2017-natgen",
#                                         outfile="test.csv")
#     # temp_list = []
#     # extract_random_lines("/Users/kanduric/Downloads/emerson-2017-natgen/HIP17737.tsv",2000,temp_list)
#     # print(len(temp_list))
