import random
import argparse
import os
from utilities_immuneml.yaml_tools import *

parser = argparse.ArgumentParser()
parser.add_argument('--simulation_yaml', help='Input simulation yaml file', required=True)
parser.add_argument('--n_lists', help='Number of lists of kmer labels to populate, where each list will have kmer '
                                      'labels increased exponentially to base 2', type=int, required=True)
parser.add_argument('--dirname', help='directory where the specification file to write', required=True)
args = parser.parse_args()


def generate_random_kmerlabel_list(kmer_list, list_size):
    try:
        sampled_list = random.sample(kmer_list, k=list_size)
        return sampled_list
    except ValueError:
        print("Error: Number of specified kmers in the supplied specification file is less than the desired number of "
              "kmers to be included in the number of lists. For e.g. if n_lists=5, at least 2, pow (n_lists-1) = 16 "
              "kmers are to be present in the file")
        sys.exit(1)



def populate_multiple_kmer_lists(sim_yaml_file, n_lists):
    simulation_yaml_obj = parse_yaml(sim_yaml_file)
    kmer_labels = simulation_yaml_obj['definitions']['motifs'].keys()
    generated_lists = [generate_random_kmerlabel_list(kmer_labels, list_size=i) for i in [pow(2, j) for j in range(n_lists)]]
    modified_yaml_dict = put_values_in_target_key(simulation_yaml_obj,'motifs',generated_lists)
    return modified_yaml_dict


def execute():
    filename = os.path.basename(args.simulation_yaml)
    filepath = os.path.join(args.dirname, filename)
    sim_yaml_obj = populate_multiple_kmer_lists(args.simulation_yaml, args.n_lists)
    write_yaml_file(sim_yaml_obj, args.dirname, filepath)

# if __name__ == "__main__":
#     populate_multiple_kmer_lists("/Users/kanduric/Documents/Projects/immuneML_test/test_yaml_specs/100_kmers_sized_4.yaml",n_lists=4)