import argparse
import pandas as pd
from scipy.stats import ttest_ind
import numpy as np
from statsmodels.stats.multitest import multipletests
import regex as re

parser = argparse.ArgumentParser()
parser.add_argument('--design_matrix_file_path', help='file path of deign matrix', required=True)
parser.add_argument('--motifs_file_path', help='file path of implanted motifs file', required=True)
args = parser.parse_args()


def regex_intersect(s1, s2):
    regexs = [re.compile(s1[0:4]), re.compile(s1[1:4] + "."), re.compile(s1[2:4] + ".."), re.compile(s1[3:4] + "..."),
              re.compile("..." + s1[0:1]), re.compile(".." + s1[0:2]), re.compile("." + s1[0:3])]
    overlaps = [4, 3, 2, 1, 1, 2, 3]
    for regex, overlap in zip(regexs, overlaps):
        m = regex.match(s2)
        if m:
            max_overlap = overlap
            break
        else:
            max_overlap = 0
    return max_overlap


def intersect_motifs_list(s1, s2_list):
    len_list = [regex_intersect(s1, motif) for motif in s2_list]
    return max(len_list)


def t_test_between_labeled_groups(design_matrix_file_path, labels_array, motifs_file_path):
    result_filename = design_matrix_file_path.split(".csv")[0] + "_t_test_results.csv"
    dat = pd.read_csv(design_matrix_file_path)
    motifs_file = pd.read_csv(motifs_file_path)
    motifs = motifs_file["motif"].tolist()
    pos = np.where(labels_array == "True")
    neg = np.where(labels_array == "False")
    column_list = [x for x in dat.columns]
    results_df = []
    for column in column_list:
        t_test_results = {"feature": column}
        group1 = dat.loc[pos[0], column]
        group2 = dat.loc[neg[0], column]
        t_test_results["pvalue"] = ttest_ind(group1, group2)[1]
        t_test_results["stdev"] = dat.loc[:, column].std()
        t_test_results["mean"] = dat.loc[:, column].mean()
        results_df.append(t_test_results)
    res = pd.DataFrame(results_df)
    res['qvalue'] = multipletests(res['pvalue'], method="fdr_bh")[1]
    res_feat = res["feature"].tolist()
    max_overlaps = [intersect_motifs_list(feature, motifs) for feature in res_feat]
    res["max_overlaps"] = max_overlaps
    res.to_csv(result_filename)


def execute():
    x = np.array(["True", "False"])
    met = np.repeat(x, 100)
    t_test_between_labeled_groups(args.design_matrix_file_path, labels_array=met,
                                  motifs_file_path=args.motifs_file_path)


if __name__ == "__main__":
    # x = np.array(["True", "False"])
    # met = np.repeat(x, 100)
    # t_test_between_labeled_groups("design_matrix.csv", labels_array=met)
    # parent_list = ["VLEI", "WKHS", "IHPI", "ELFW", "QLLK", "HWNI", "IHKY", "QKRY", "DLTD", "CEAH", "WMLV", "VPGK",
    #                "QQEP", "HSWF", "KCDG", "VAPH", "RNYN", "CPKH", "TVIR", "FMYF", "VNKC", "REDK", "GHRK", "PSGK",
    #                "MQIN", "TCCP", "SDLW", "NMVE", "WNNI", "QDNV", "RPDP", "ANKI", "VCVH", "RYVP", "YEVT", "EHWA",
    #                "SAKW", "MAPT", "LFFD", "NHCC", "LGEC", "FDKH", "GKTQ", "GNCN", "SDGC", "FKSC", "HFEY", "SWYD",
    #                "GGYP", "THIE", "WVIP", "VAAL", "VAVK", "PSIK", "GQRA", "IVKA", "TTCG", "PQHC", "DVDQ", "ILGE",
    #                "QCEG", "SGPG", "ECGG", "FSHP"]
    # print(intersect_motifs_list("ZZZZ", parent_list))
