import sys
import yaml
import os


def parse_yaml(text_file):
    with open(text_file, "r") as yaml_file:
        try:
            yaml_obj = yaml.load(yaml_file, Loader=yaml.FullLoader)
            assert yaml_obj is not None, "The supplied yaml file," + text_file + ", is empty"
        except Exception as e:
            print("Error: that looks like an invalid yaml file. Consider validating your yaml file using one of the "
                  "online yaml validators; for instance: https://jsonformatter.org/yaml-validator")
            print("Exception: %s" % str(e))
            sys.exit(1)
    return yaml_obj


def get_target_key_path(yaml_dict, target_key):
    try:
        for key, value in yaml_dict.items():
            if isinstance(value, dict):
                path = get_target_key_path(value, target_key)
                if path:
                    return [key] + path
            elif key == target_key and isinstance(value, list):
                return [key]
    except Exception as e:
        print("Exception: %s" % str(e))
        sys.exit(1)


def put_values_in_target_key(yaml_dict, target_key, insert_value):
    for key, value in yaml_dict.items():
        if isinstance(value, dict):
            put_values_in_target_key(value, target_key, insert_value)
        elif key == target_key and isinstance(value, list):
            yaml_dict[key] = insert_value
    return yaml_dict


def write_yaml_file(simulation_yaml_obj, dirname, filename):
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    with open(filename, "w+") as yaml_file:
        yaml.dump(simulation_yaml_obj, yaml_file)